@@ -1,18 +1,3 @@
-/*
- * Copyright 2021 Actyx AG
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
 use serde::{de::Error, Deserialize, Deserializer};
 use unicode_normalization::UnicodeNormalization;
 
@@ -90,9 +75,6 @@ macro_rules! mk_scalar {
             pub fn as_str(&self) -> &str {
                 &self.0
             }
-            pub fn as_arc(&self) -> &::std::sync::Arc<str> {
-                &self.0.as_arc()
-            }
         }
 
         impl ::std::convert::TryFrom<&str> for $id {
