@@ -1,18 +1,3 @@
-/*
- * Copyright 2021 Actyx AG
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
 use anyhow::Result;
 use async_trait::async_trait;
 use futures::stream::BoxStream;
@@ -117,25 +102,24 @@ impl<T> From<Event<T>> for EventResponse<T> {
     }
 }
 
-#[cfg(test)]
 impl EventResponse<Payload> {
     /// Try to extract the given type from the generic payload and return a new
     /// event envelope if successful. The produced payload is deserialized as efficiently
     /// as possible and may therefore still reference memory owned by the `Payload`.
     /// You may need to `.clone()` it to remove this dependency.
-    pub fn extract<'a, T>(&'a self) -> EventResponse<T>
+    pub fn extract<'a, T>(&'a self) -> Result<EventResponse<T>, serde_cbor::Error>
     where
         T: Deserialize<'a> + Clone,
     {
-        EventResponse {
+        Ok(EventResponse {
             stream: self.stream,
             lamport: self.lamport,
             offset: self.offset,
             timestamp: self.timestamp,
             tags: self.tags.clone(),
             app_id: self.app_id.clone(),
-            payload: self.payload.extract::<T>().unwrap(),
-        }
+            payload: self.payload.extract::<T>()?,
+        })
     }
 }
 
