@@ -1,18 +1,3 @@
-/*
- * Copyright 2021 Actyx AG
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
 use std::{
     collections::BTreeSet,
     convert::TryFrom,
@@ -115,6 +100,18 @@ impl Tag {
     pub fn len(&self) -> usize {
         self.0.len()
     }
+    /// Constructs an `TagSet` with \<tag> and \<tag>:\<id>
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// use actyx_sdk::{TagSet, tags, tag};
+    /// let tags: TagSet = tag!("machine").with_id("Drill-404");
+    /// assert_eq!(tags, tags!("machine", "machine:Drill-404"));
+    /// ```
+    pub fn with_id(self, id: impl Into<String>) -> TagSet {
+        vec![self.clone(), self + format!(":{}", id.into())].into()
+    }
 }
 
 impl TryFrom<&str> for Tag {
@@ -516,4 +513,9 @@ mod tests {
 
         assert_eq!(vec![a, b, c].into_iter().collect::<TagSet>(), tags!("a", "b", "c"));
     }
+
+    #[test]
+    fn tag_with_id() {
+        assert_eq!(tag!("a").with_id("b"), tags!("a", "a:b"));
+    }
 }
