@@ -1,18 +1,3 @@
-/*
- * Copyright 2021 Actyx AG
- *
- * Licensed under the Apache License, Version 2.0 (the "License");
- * you may not use this file except in compliance with the License.
- * You may obtain a copy of the License at
- *
- *     http://www.apache.org/licenses/LICENSE-2.0
- *
- * Unless required by applicable law or agreed to in writing, software
- * distributed under the License is distributed on an "AS IS" BASIS,
- * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * See the License for the specific language governing permissions and
- * limitations under the License.
- */
 use fixed::types::extra::*;
 use fixed::{
     traits::{FromFixed, ToFixed},
@@ -417,13 +402,26 @@ mod tests {
     #[cfg(feature = "dataflow")]
     #[rustfmt::skip]
     fn expected_bits() -> Vec<u8> {
-        vec![
-            0b100__00011, 0b1_11001_00, 0b1101_1001, 0b101_1, 0, 0, 0, 0, //
-            0, 0, 0, 0, 0, 0, 0, 0, //
-            1, 0, 0, 0, 0, 0, 0, 0, // Option == Some
-            0b100_10001, 0b11__00, 0, 0, 0, 0, 0, 0, //
-            0, 0, 0, 0, 0, 0, 0, 0, //
-        ]
+        if cfg!(target_arch = "x86_64") {
+            vec![
+                0b100__00011, 0b1_11001_00, 0b1101_1001, 0b101_1, 0, 0, 0, 0, //
+                0, 0, 0, 0, 0, 0, 0, 0, //
+                1, 0, 0, 0, 0, 0, 0, 0, // Option == Some
+                0b100_10001, 0b11__00, 0, 0, 0, 0, 0, 0, //
+                0, 0, 0, 0, 0, 0, 0, 0, //
+            ]
+        } else if cfg!(target_arch = "aarch64") {
+            vec![
+                0b100__00011, 0b1_11001_00, 0b1101_1001, 0b101_1, 0, 0, 0, 0, //
+                0, 0, 0, 0, 0, 0, 0, 0, //
+                1, 0, 0, 0, 0, 0, 0, 0, // Option == Some
+                0, 0, 0, 0, 0, 0, 0, 0, //
+                0b100_10001, 0b11__00, 0, 0, 0, 0, 0, 0, //
+                0, 0, 0, 0, 0, 0, 0, 0, //
+            ]
+        } else {
+            panic!("unsupported CPU architecture")
+        }
     }
 
     #[test]
@@ -503,7 +501,6 @@ mod tests {
         assert_eq!(v.iter().product::<FixNum<U10>>(), FixNum::panicking(6));
         assert_eq!(v.clone().into_iter().product::<FixNum<U10>>(), FixNum::panicking(6));
 
-        let v = v.iter().map(|x| x + 3).collect::<Vec<_>>();
-        assert_eq!(v.into_iter().sum::<FixNum<U10>>(), FixNum::panicking(15));
+        assert_eq!(v.iter().map(|x| x + 3).sum::<FixNum<U10>>(), FixNum::panicking(15));
     }
 }
